#include "testApp.h"



//--------------------------------------------------------------
void testApp::setup(){

	ofBackground(255,255,255);

	// 2 output channels,
	// 0 input channels
	// 44100 samples per second
	// 256 samples per buffer
	// 4 num buffers (latency)

	ofSoundStreamSetup(2,0,this, 44100,256, 4);

	ofSetFrameRate(60);
	
	frequency = 440;
	position = 0;
	positionAdder = (frequency / 44100.0) * TWO_PI;
	
	envelopeLength = 2; //run the envelope over a two second span
	envelopePct = 0;
	envelopePctAdder = 1.0/(44100.0*envelopeLength); // how many samples per second * number of seconds
	
	panel.setup("control", 770, 0, 300, 400);
	panel.addPanel("sound settings", 1, false);
	panel.setWhichPanel("sound settings");
	
	panel.addSlider("Envelope Size", "VAL_SIZE", 32,2,400, true);
	panel.addSlider("Envelope Length", "VAL_LENGTH", 2,0.01,2, false);
	panel.loadSettings("settings.xml");
}


//--------------------------------------------------------------
void testApp::update(){
	panel.update();
	
	if(envelopeLength !=  panel.getValueF("VAL_LENGTH") ){
		envelopeLength = panel.getValueF("VAL_LENGTH");
		envelopePct = 0;
		envelopePctAdder = 1.0/(44100.0*envelopeLength); // how many samples per second * number of seconds
	}
	
	myEnvelope.setup(panel.getValueF("VAL_SIZE"));
}

//--------------------------------------------------------------
void testApp::draw(){
	panel.draw();
	
	ofSetColor(0,0,0);
	ofDrawBitmapString("frequency = " + ofToString(frequency), 50, 500);
	
	
	ofNoFill();
	ofSetLineWidth(2);
	myEnvelope.draw(10,110,400,100);

	ofSetColor(255,0,0);
	ofSetLineWidth(1);
	ofLine(10 + envelopePct*400, 10, 10 + envelopePct*400, 110); // draw a line to show where we are going through the envelope
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){
	
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	panel.mouseDragged(x,y,button);
	
	if(x>=10 && x<410 && y>0 && y<110) // we are within the envelope drawing
	{
		myEnvelope.setTableValue(int(((float)x-10)/400*myEnvelope.getTableSize()), 1.0-(float)y/110);
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	panel.mousePressed(x,y,button);
	
	if(x>=10 && x<410 && y>0 && y<110) // we are within the envelope drawing
	{
		myEnvelope.setTableValue(int(((float)x-10)/400*myEnvelope.getTableSize()), 1.0-(float)y/110);
	}
}


//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	panel.mouseReleased();
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}
//--------------------------------------------------------------
void testApp::audioRequested 	(float * output, int bufferSize, int nChannels){

	for (int i = 0; i < bufferSize; i++){
		
		float sample = sin( position );
		position += positionAdder;
		
		sample*=myEnvelope.getEnvelopeValue(envelopePct);
		
		output[i * 2 + 0] = sample;			// left channel
		output[i * 2 + 1] = sample;			// right channel
		
		envelopePct += envelopePctAdder;
				
		if(envelopePct > 1.0) // if we are past the end of the envelope, loop back to the start
			envelopePct -= 1.0;
		
	}
	
	// because sin(..) of very large values is less precise, let's keep the position variable in the range of 0 to TWO_PI.  it's cyclical
	// so doing something like this, is very similar to %= for integer numbers. 
	while (position > TWO_PI) {
		position -= TWO_PI;	
	}
	
}

